=== modified file 'bzrlib/tests/test_transport.py'
--- bzrlib/tests/test_transport.py	2011-07-16 20:12:14 +0000
+++ bzrlib/tests/test_transport.py	2011-07-16 20:24:37 +0000
@@ -1012,3 +1012,11 @@
         directories.register("bar:", SomeDirectory, "Dummy directory")
         self.addCleanup(directories.remove, "bar:")
         self.assertEquals("http://bar", location_to_url("bar:"))
+
+    def test_unicode_url(self):
+        self.assertRaises(errors.InvalidURL, location_to_url,
+            "http://fo/\xc3\xaf".decode("utf-8"))
+
+    def test_unicode_path(self):
+        self.assertEquals("file:///foo/bar%C3%AF",
+            location_to_url("/foo/bar\xc3\xaf".decode("utf-8")))

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2011-07-16 20:06:11 +0000
+++ bzrlib/transport/__init__.py	2011-07-16 20:24:37 +0000
@@ -1566,6 +1566,16 @@
         raise AssertionError("location not a byte or unicode string")
     from bzrlib.directory_service import directories
     location = directories.dereference(location)
+
+    # Catch any URLs which are passing Unicode rather than ASCII
+    try:
+        location = location.encode('ascii')
+    except UnicodeError:
+        if urlutils.is_url(location):
+            raise errors.InvalidURL(path=location,
+                extra='URLs must be properly escaped')
+        location = urlutils.local_path_to_url(location)
+
     return location
 
 
@@ -1585,25 +1595,6 @@
     last_err = None
     base = location_to_url(base)
 
-    def convert_path_to_url(base, error_str):
-        if urlutils.is_url(base):
-            # This looks like a URL, but we weren't able to
-            # instantiate it as such raise an appropriate error
-            # FIXME: we have a 'error_str' unused and we use last_err below
-            raise errors.UnsupportedProtocol(base, last_err)
-        # This doesn't look like a protocol, consider it a local path
-        new_base = urlutils.local_path_to_url(base)
-        # mutter('converting os path %r => url %s', base, new_base)
-        return new_base
-
-    # Catch any URLs which are passing Unicode rather than ASCII
-    try:
-        base = base.encode('ascii')
-    except UnicodeError:
-        # Only local paths can be Unicode
-        base = convert_path_to_url(base,
-            'URLs must be properly escaped (protocol: %s)')
-
     transport = None
     if possible_transports is not None:
         for t in possible_transports:
@@ -1626,7 +1617,12 @@
 
     # We tried all the different protocols, now try one last time
     # as a local protocol
-    base = convert_path_to_url(base, 'Unsupported protocol: %s')
+    if urlutils.is_url(base):
+        # This looks like a URL, but we weren't able to
+        # instantiate it as such. raise an appropriate error
+        raise errors.UnsupportedProtocol(base, last_err)
+    # This doesn't look like a protocol, consider it a local path
+    base = urlutils.local_path_to_url(base)
 
     # The default handler is the filesystem handler, stored as protocol None
     factory_list = transport_list_registry.get(None)

